@@ -63,7 +63,10 @@ class JinjaTracer:
         self.program_counter: int = 0
         self.source_idx: int = 0
 
-    def trace(self, append_to_templated: str = "") -> JinjaTrace:
+    def trace(
+        self,
+        append_to_templated: str = "",
+    ) -> JinjaTrace:
         """Executes raw_str. Returns template output and trace."""
         trace_template_str = "".join(
             cast(str, self.raw_slice_info[rs].alternate_code)
@@ -142,8 +145,24 @@ class JinjaTracer:
             )
         return raw_slices_search_result[0]
 
-    def move_to_slice(self, target_slice_idx: int, target_slice_length: int) -> None:
-        """Given a template location, walk execution to that point."""
+    def move_to_slice(
+        self,
+        target_slice_idx: int,
+        target_slice_length: int,
+    ) -> Dict[int, List[int]]:
+        """Given a template location, walk execution to that point.
+
+        This updates the internal `program_counter` to the appropriate
+        location.
+
+        Returns:
+            :obj:`dict`: For each step in the template, a :obj:`list` of
+                which steps are accessible. In many cases each step will
+                only have one accessible next step (the following one),
+                however for branches in the program there may be more than
+                one.
+        """
+        step_candidates = {}
         while self.program_counter < len(self.raw_sliced):
             self.record_trace(
                 target_slice_length if self.program_counter == target_slice_idx else 0
@@ -153,22 +172,28 @@ class JinjaTracer:
                 # Reached the target slice. Go to next location and stop.
                 self.program_counter += 1
                 break
-            else:
-                # Choose the next step.
 
-                # We could simply go to the next slice (sequential execution).
-                candidates = [self.program_counter + 1]
-                # If we have other options, consider those.
-                for next_slice_idx in self.raw_slice_info[
-                    current_raw_slice
-                ].next_slice_indices:
-                    # It's a valid possibility if it does not take us past the
-                    # target.
-                    if next_slice_idx <= target_slice_idx:
-                        candidates.append(next_slice_idx)
-                # Choose the candidate that takes us closest to the target.
-                candidates.sort(key=lambda c: abs(target_slice_idx - c))
-                self.program_counter = candidates[0]
+            # Choose the next step.
+            # We could simply go to the next slice (sequential execution).
+            candidates = [self.program_counter + 1]
+            # If we have other options, consider those.
+            candidates.extend(
+                filter(
+                    # They're a valid possibility if
+                    # they don't take us past the target.
+                    lambda idx: idx <= target_slice_idx,
+                    self.raw_slice_info[current_raw_slice].next_slice_indices,
+                )
+            )
+            # Choose the candidate that takes us closest to the target.
+            candidates.sort(key=lambda c: abs(target_slice_idx - c))
+            # Save all the candidates for each step so we can return them later.
+            step_candidates[self.program_counter] = candidates
+            # Step forward to the best step found.
+            self.program_counter = candidates[0]
+
+        # Return the candidates at each step.
+        return step_candidates
 
     def record_trace(
         self,
@@ -176,7 +201,15 @@ class JinjaTracer:
         slice_idx: Optional[int] = None,
         slice_type: Optional[str] = None,
     ) -> None:
-        """Add the specified (default: current) location to the trace."""
+        """Add the specified (default: current) location to the trace.
+
+        Args:
+            target_slice_length (int): The length of the target slice.
+            slice_idx (Optional[int], optional): The index of the slice.
+            Defaults to None.
+            slice_type (Optional[str], optional): The type of the slice.
+            Defaults to None.
+        """
         if slice_idx is None:
             slice_idx = self.program_counter
         if slice_type is None:
@@ -361,12 +394,7 @@ class JinjaAnalyzer:
 
         # https://jinja.palletsprojects.com/en/2.11.x/api/#jinja2.Environment.lex
         block_idx = 0
-        last_elem_type = None
         for _, elem_type, raw in self.env.lex(self.raw_str):
-            if last_elem_type == "block_end" or elem_type == "block_start":
-                block_idx += 1
-            last_elem_type = elem_type
-
             if elem_type == "data":
                 self.track_literal(raw, block_idx)
                 continue
@@ -385,7 +413,7 @@ class JinjaAnalyzer:
             m_close = None
             if elem_type.endswith("_end") or elem_type == "raw_begin":
                 block_type = self.block_types[elem_type]
-                block_subtype = None
+                block_tag = None
                 # Handle starts and ends of blocks
                 if block_type in ("block", "templated"):
                     m_open = self.re_open_tag.search(str_parts[0])
@@ -396,9 +424,8 @@ class JinjaAnalyzer:
                         )
 
                     if block_type == "block" and tag_contents:
-                        block_type, block_subtype = self.extract_block_type(
-                            tag_contents[0], block_subtype
-                        )
+                        block_type = self.extract_block_type(tag_contents[0])
+                        block_tag = tag_contents[0]
                     if block_type == "templated" and tag_contents:
                         assert m_open and m_close
                         raw_slice_info = self.track_templated(
@@ -415,6 +442,8 @@ class JinjaAnalyzer:
                 m_strip_right = regex.search(
                     r"\s+$", raw, regex.MULTILINE | regex.DOTALL
                 )
+                if block_type == "block_start":
+                    block_idx += 1
                 if elem_type.endswith("_end") and raw.startswith("-") and m_strip_right:
                     # Right whitespace was stripped after closing block. Split
                     # off the trailing whitespace into a separate slice. The
@@ -429,19 +458,20 @@ class JinjaAnalyzer:
                             str_buff[:-trailing_chars],
                             block_type,
                             self.idx_raw,
-                            block_subtype,
                             block_idx,
+                            block_tag,
                         )
                     )
                     self.raw_slice_info[self.raw_sliced[-1]] = raw_slice_info
                     slice_idx = len(self.raw_sliced) - 1
                     self.idx_raw += len(str_buff) - trailing_chars
+                    if block_type == "block_end":
+                        block_idx += 1
                     self.raw_sliced.append(
                         RawFileSlice(
                             str_buff[-trailing_chars:],
                             "literal",
                             self.idx_raw,
-                            None,
                             block_idx,
                         )
                     )
@@ -455,13 +485,15 @@ class JinjaAnalyzer:
                             str_buff,
                             block_type,
                             self.idx_raw,
-                            block_subtype,
                             block_idx,
+                            block_tag,
                         )
                     )
                     self.raw_slice_info[self.raw_sliced[-1]] = raw_slice_info
                     slice_idx = len(self.raw_sliced) - 1
                     self.idx_raw += len(str_buff)
+                    if block_type == "block_end":
+                        block_idx += 1
                 if block_type.startswith("block"):
                     self.track_block_end(block_type, tag_contents[0])
                     self.update_next_slice_indices(
@@ -483,7 +515,18 @@ class JinjaAnalyzer:
         m_close: regex.Match[str],
         tag_contents: List[str],
     ) -> RawSliceInfo:
-        """Compute tracking info for Jinja templated region, e.g. {{ foo }}."""
+        """Compute tracking info for Jinja templated region, e.g. {{ foo }}.
+
+        Args:
+            m_open (regex.Match): A regex match object representing the opening tag.
+            m_close (regex.Match): A regex match object representing the closing tag.
+            tag_contents (List[str]): A list of strings representing the contents of the
+                tag.
+
+        Returns:
+            RawSliceInfo: A RawSliceInfo object containing the computed
+            tracking info.
+        """
         unique_alternate_id = self.next_slice_id()
         open_ = m_open.group(1)
         close_ = m_close.group(1)
@@ -501,7 +544,18 @@ class JinjaAnalyzer:
         m_close: regex.Match[str],
         tag_contents: List[str],
     ) -> RawSliceInfo:
-        """Set up tracking for "{% call ... %}"."""
+        """Set up tracking for "{% call ... %}".
+
+        Args:
+            m_open (regex.Match): A regex match object representing the opening tag.
+            m_close (regex.Match): A regex match object representing the closing tag.
+            tag_contents (List[str]): A list of strings representing the contents of the
+                tag.
+
+        Returns:
+            RawSliceInfo: A RawSliceInfo object containing the computed
+            tracking info.
+        """
         unique_alternate_id = self.next_slice_id()
         open_ = m_open.group(1)
         close_ = m_close.group(1)
@@ -520,7 +574,6 @@ class JinjaAnalyzer:
                 raw,
                 "literal",
                 self.idx_raw,
-                None,
                 block_idx,
             )
         )
@@ -531,9 +584,7 @@ class JinjaAnalyzer:
         self.idx_raw += len(raw)
 
     @staticmethod
-    def extract_block_type(
-        tag_name: str, block_subtype: Optional[str] = None
-    ) -> Tuple[str, Optional[str]]:
+    def extract_block_type(tag_name: str) -> str:
         """Determine block type."""
         # :TRICKY: Syntactically, the Jinja {% include %} directive looks like
         # a block, but its behavior is basically syntactic sugar for
@@ -548,9 +599,7 @@ class JinjaAnalyzer:
             block_type = "block_mid"
         else:
             block_type = "block_start"
-            if tag_name == "for":
-                block_subtype = "loop"
-        return block_type, block_subtype
+        return block_type
 
     @staticmethod
     def extract_tag_contents(
@@ -562,6 +611,15 @@ class JinjaAnalyzer:
         """Given Jinja tag info, return the stuff inside the braces.
 
         I.e. Trim off the brackets and the whitespace.
+
+        Args:
+            str_parts (List[str]): A list of string parts.
+            m_close (regex.Match[str]): The regex match for the closing tag.
+            m_open (regex.Match[str]): The regex match for the opening tag.
+            str_buff (str): The string buffer.
+
+        Returns:
+            List[str]: The trimmed parts inside the Jinja tag.
         """
         if len(str_parts) >= 3:
             # Handle a tag received as individual parts.
@@ -577,7 +635,12 @@ class JinjaAnalyzer:
         return trimmed_parts
 
     def track_block_end(self, block_type: str, tag_name: str) -> None:
-        """On ending a 'for' or 'if' block, set up tracking."""
+        """On ending a 'for' or 'if' block, set up tracking.
+
+        Args:
+            block_type (str): The type of block ('for' or 'if').
+            tag_name (str): The name of the tag.
+        """
         if block_type == "block_end" and tag_name in (
             "endfor",
             "endif",
@@ -601,11 +664,16 @@ class JinjaAnalyzer:
             "if",
         ):
             self.stack.append(slice_idx)
-        elif block_type == "block_mid":
+            return None
+        elif not self.stack:
+            return None
+
+        _idx = self.stack[-1]
+        _raw_slice = self.raw_sliced[_idx]
+        _slice_info = self.raw_slice_info[_raw_slice]
+        if block_type == "block_mid":
             # Record potential forward jump over this block.
-            self.raw_slice_info[
-                self.raw_sliced[self.stack[-1]]
-            ].next_slice_indices.append(slice_idx)
+            _slice_info.next_slice_indices.append(slice_idx)
             self.stack.pop()
             self.stack.append(slice_idx)
         elif block_type == "block_end" and tag_name in (
@@ -614,15 +682,13 @@ class JinjaAnalyzer:
         ):
             if not self.inside_set_macro_or_call:
                 # Record potential forward jump over this block.
-                self.raw_slice_info[
-                    self.raw_sliced[self.stack[-1]]
-                ].next_slice_indices.append(slice_idx)
-                if self.raw_sliced[self.stack[-1]].slice_subtype == "loop":
+                _slice_info.next_slice_indices.append(slice_idx)
+                self.stack.pop()
+                if _raw_slice.slice_type == "block_start" and _raw_slice.tag == "for":
                     # Record potential backward jump to the loop beginning.
                     self.raw_slice_info[
                         self.raw_sliced[slice_idx]
-                    ].next_slice_indices.append(self.stack[-1] + 1)
-                self.stack.pop()
+                    ].next_slice_indices.append(_idx + 1)
 
     def handle_left_whitespace_stripping(self, token: str, block_idx: int) -> None:
         """If block open uses whitespace stripping, record it.
@@ -665,7 +731,7 @@ class JinjaAnalyzer:
             )
         # Treat the skipped whitespace as a literal.
         self.raw_sliced.append(
-            RawFileSlice(skipped_str, "literal", self.idx_raw, None, block_idx)
+            RawFileSlice(skipped_str, "literal", self.idx_raw, block_idx)
         )
         self.raw_slice_info[self.raw_sliced[-1]] = self.slice_info_for_literal(0)
         self.idx_raw += num_chars_skipped
