@@ -1,5 +1,5 @@
 """Implementation of Rule RF05."""
-from typing import Optional, Set, List
+from typing import List, Optional, Set
 
 import regex
 
